import unittest
from bewe.backtest import holder
from bewe.backtest import backtester as bt


class TestBacktester(unittest.TestCase):

    def test_create(self):
        bank = holder.Bank(1000000)
        trader = bt.BackTester(bank)
        self.assertIsInstance(trader, bt.BackTester)


if __name__ == '__main__':
    unittest.main()



import datetime
import sqlalchemy
from bewe.services.databases import db_manager
from sqlalchemy import orm


class Base(orm.DeclarativeBase):
    pass


class Stock(Base):
    __tablename__ = 'stock'

    stock_id = sqlalchemy.Column(sqlalchemy.String(50), primary_key=True)
    stock_name = sqlalchemy.Column(sqlalchemy.String(60))
    flag = sqlalchemy.Column(sqlalchemy.Boolean, default=True)
    last_execution = sqlalchemy.Column(sqlalchemy.DateTime, nullable=True)
    created_at = sqlalchemy.Column(sqlalchemy.DateTime, default=datetime.datetime.now)
    updated_at = sqlalchemy.Column(
        sqlalchemy.DateTime, default=datetime.datetime.now, onupdate=datetime.datetime.now)

    def __init__(self, stock_id, stock_name):
        self.stock_id = stock_id
        self.stock_name = stock_name


class Category(Base):
    __tablename__ = 'stock_category'

    stock_id = sqlalchemy.Column(
        sqlalchemy.String(50), sqlalchemy.ForeignKey('stock.stock_id'), nullable=False, primary_key=True)
    category = sqlalchemy.Column(sqlalchemy.String(30), primary_key=True)
    created_at = sqlalchemy.Column(sqlalchemy.DateTime, default=datetime.datetime.now)
    updated_at = sqlalchemy.Column(
        sqlalchemy.DateTime, default=datetime.datetime.now, onupdate=datetime.datetime.now)

    def __init__(self, stock_id, category):
        self.stock_id = stock_id
        self.category = category


class StockPrice(Base):
    __tablename__ = 'stock_price'

    stock_id = sqlalchemy.Column(
        sqlalchemy.String(50), sqlalchemy.ForeignKey('stock.stock_id'), nullable=False, primary_key=True)
    open_price = sqlalchemy.Column(sqlalchemy.Float(8))
    close_price = sqlalchemy.Column(sqlalchemy.Float(8))
    max_price = sqlalchemy.Column(sqlalchemy.Float(8))
    min_price = sqlalchemy.Column(sqlalchemy.Float(8))
    volume = sqlalchemy.Column(sqlalchemy.Float(8))
    created_at = sqlalchemy.Column(sqlalchemy.DateTime, default=datetime.datetime.now)
    updated_at = sqlalchemy.Column(
        sqlalchemy.DateTime, default=datetime.datetime.now, onupdate=datetime.datetime.now)


def create_delete_all():
    db = db_manager.DBManager()
    engine = db.engine
    Base.metadata.drop_all(engine)
    Base.metadata.create_all(engine)


if __name__ == '__main__':
    create_delete_all()




import dataclasses
import pandas as pd
from typing import Any, Mapping, Optional, Sequence


@dataclasses.dataclass(frozen=True)
class DataFormat:
    required_field: Sequence[str] = dataclasses.field(default_factory=list)


@dataclasses.dataclass
class DataUnit:
    format: DataFormat
    data: Mapping[Any, Any]
    actionable_price: float

    def __post_init__(self):
        for col in self.format.required_field:
            if col not in self.data:
                raise KeyError(f'{col} is required for the target unit.')
            self.__setattr__(col, self.data[col])


class DataContainer:
    name = 'BaseContainer'

    def __init__(self, asset_name: str, data: Optional[Sequence[DataUnit]] = None):
        self.asset_name = asset_name
        self.data: Optional[Sequence[DataUnit]] = data
        self.index = 0
        self.size = 0 if not self.data else len(self.data)

    def __iter__(self) -> 'DataContainer':
        return self

    def __next__(self) -> DataUnit:
        if self.index == self.size:
            raise StopIteration

        if not self.data:
            raise StopIteration

        unit = self.data[self.index]
        self.index += 1
        return unit

    def __len__(self):
        return 0 if not self.data else len(self.data)
    
    def transform(self, data: pd.DataFrame):
        raise NotImplementedError
    
    def get_format(self) -> DataFormat:
        raise NotImplementedError



import requests
from bewe.services.crawler import secret_manager
from bewe.services.databases import asset_orm
from bewe.services.databases import db_manager
from FinMind.data import data_loader

_FINMIND_KEY = 'FINMIND_API'


class FinmindWrapper:
    def __init__(self):
        self.token = secret_manager.get_secret_tokens(_FINMIND_KEY)
        self.data_loader = data_loader.DataLoader()
        self.data_loader.login_by_token(self.token)
        self.db_manager = db_manager.DBManager()

    def get_tw_stock_info(self):
        df = self.data_loader.taiwan_stock_info()

        stocks, categories = [], []
        for _, row in df.iterrows():
            stock = asset_orm.Stock(
                stock_id=row.stock_id,
                stock_name=row.stock_name
            )

            category = asset_orm.Category(
                stock_id=row.stock_id,
                category=row.industry_category
            )

            stocks.append(stock)
            categories.append(category)

        self.db_manager.update(stocks)
        self.db_manager.update(categories)





import unittest

from bewe.alpha_seeker import base_data


class TestBaseData(unittest.TestCase):

    def get_mock_units(self):
        data_format = base_data.DataFormat(
            required_field=['open', 'close']
        )

        return [
            base_data.DataUnit(
                format=data_format,
                data={'open': 100, 'close': 50},
                actionable_price=80
            ),
            base_data.DataUnit(
                format=data_format,
                data={'open': 80, 'close': 100},
                actionable_price=80
            )
        ]

    def test_iter(self):
        asset_name = 'mock'
        mock_data = self.get_mock_units()
        container = base_data.DataContainer(asset_name, data=mock_data)
        iter_units = [unit for unit in container]
        self.assertEqual(len(iter_units), 2)
        self.assertListEqual(iter_units, mock_data)

    def test_empty_iter(self):
        asset_name = 'mock'
        container = base_data.DataContainer(asset_name)
        iter_units = [unit for unit in container]
        self.assertEqual(len(iter_units), 0)

    def test_data_unit_miss_keys(self):
        required = ['open', 'close']
        dformat = base_data.DataFormat(required)

        with self.assertRaises(KeyError):
            base_data.DataUnit(
                format=dformat,
                data={'open': 100},
                actionable_price=50
            )

    def test_data_unit(self):
        required = ['open', 'close']
        dformat = base_data.DataFormat(required)
        data = {'open': 100, 'close': 50}
        unit = base_data.DataUnit(
            format=dformat,
            data=data,
            actionable_price=100
        )

        self.assertEqual(unit.open, 100)
        self.assertEqual(unit.close, 50)


if __name__ == '__main__':
    unittest.main()



from bewe.alpha_seeker import base_strategy
from bewe.alpha_seeker import base_data
from bewe.backtest import holder


class BackTester:
    def __init__(self, bank: holder.Bank):
        self.bank = bank

    def backtest(self, strategy: base_strategy.Strategy, container: base_data.DataContainer):
        for unit in container:
            execution_plan = strategy.predict(unit)
            self.bank.act(execution_plan, container.asset_name, unit.actional_price)



import unittest
import parameterized
from bewe.backtest import holder
from bewe.alpha_seeker import base_strategy


class BankTest(unittest.TestCase):

    def setUp(self) -> None:
        self.bank = holder.Bank(1000000)

    def test_create_bank(self):
        bank = holder.Bank(50000)
        self.assertEqual(bank.budget, 50000)

    @parameterized.parameterized.expand([
        (0.4, 2.0, 10000, 1000.0),
        (0.5, 1.2, 50000, 4167.0),
        (0.2, 1.2, 10000, 0.0)
    ])
    def test_kelly_formula(self, conf, gain, budget, expected):
        actual = self.bank._kelly_formula(confidence=conf, gain=gain, budget=budget)
        self.assertEqual(expected, actual)

    @parameterized.parameterized.expand([
        (0.5, 1.5, 50, 3000, 850000),
        (0.2, 1.5, 50, 0, 1000000)
    ])
    def test_act(self, conf, gain, target_price, expected_volume, expected_budget):
        asset_name = 'mock'
        execution = base_strategy.Execution(
            action=base_strategy.Action.BUY,
            confidence=conf,
            gain=gain
        )

        self.bank.act(execution, asset_name, target_price)

        self.assertTrue(asset_name in self.bank.asset)
        self.assertEqual(self.bank.asset[asset_name].volume, expected_volume)
        self.assertEqual(self.bank.budget, expected_budget)


if __name__ == '__main__':
    unittest.main()



import dataclasses
from bewe.alpha_seeker import base_strategy
from typing import Any, Dict, Optional, Sequence, Tuple

_MIN_GAIN_RATE = 0.1
_MIN_WIN_RATE = 0.05


@dataclasses.dataclass
class Holder:
    name: Any
    hold_volumes: Sequence[float] = dataclasses.field(default_factory=list)
    hold_prices: Sequence[float] = dataclasses.field(default_factory=list)

    @property
    def volume(self) -> float:
        return sum(self.hold_volumes)

    @property
    def total_cost(self) -> float:
        total = 0.0
        for price, volume in zip(self.hold_prices, self.hold_volumes):
            total += price * volume
        return abs(total)
            
    @property
    def avg_cost(self) -> float:
        if self.volume == 0:
            return 0.0
        return self.total_cost / self.volume
        
    def current_profit(self, current_price: float):
        return self.volume * current_price - self.total_cost
        
    def current_value(self, current_price: float):
        return self.cost + self.current_profit(current_price)
    
    def execute(self, price: float, volume: float, lever: float = 1.0):
        self.hold_volumes.append(volume)
        self.hold_prices.append(price)


class Bank:
    def __init__(self, budget: float, win_rate: float = 0.3, gain_rate: float = 0.2):
        self.budget = budget
        self.init_value = budget
        self.asset: Dict[str, Holder] = {}
        self.history: Sequence[Tuple[base_strategy.Execution, float]] = []
        self.win_rate = win_rate
        self.gain_rate = gain_rate
        self.sell_record = [win_rate]
        self.profit_record = [gain_rate]

    def _kelly_formula(
            self,
            confidence: Optional[float] = None, 
            budget: Optional[float] = None,
            gain: Optional[float] = None) -> float:

        c = self.win_rate if not confidence else confidence
        b = self.budget if not budget else budget
        g = self.gain_rate if not gain else gain

        if g < 1.0:
            raise ValueError(f'Gain rate {g} is lower than 1.0.')
        kelly_value = round(b * (g * c + c - 1) / g)
        return max(kelly_value, 0.0)

    def review(self) -> None:
        self.win_rate = max(sum(self.sell_record) / len(self.sell_record), _MIN_WIN_RATE)
        self.gain_rate = max(sum(self.profit_record) / len(self.profit_record), _MIN_GAIN_RATE)

    def act(self, execution: base_strategy.Execution, asset_name: str, target: float, volume: float = 0) -> None:

        if asset_name not in self.asset:
            self.asset[asset_name] = Holder(asset_name)
        
        asset: Holder = self.asset[asset_name]
        
        if execution.action == base_strategy.Action.BUY:
            default_units = self._kelly_formula(confidence=execution.confidence, gain=execution.gain) // target
            possible_units = default_units if not volume else volume

        elif execution.action == base_strategy.Action.SELL:
            asset_current_value = asset.current_value
            default_units = self._kelly_formula(
                confidence=execution.confidence,
                budget=asset_current_value,
                gain=execution.gain) // target
            possible_units = - default_units if not volume else volume

            review_record = 1 if target > asset.avg_cost else 0
            profit_ratio = target / asset.avg_cost - 1

            self.sell_record.append(review_record)
            self.profit_record.append(profit_ratio)
            
        else:
            possible_units = 0.0

        possible_units = (possible_units//1000) * 1000

        if possible_units < 0.0 and execution.action != base_strategy.Action.SELL:
            raise ValueError('Action should be Buy but the target is negative.')

        self.budget -= possible_units * target
        self.history.append((execution, possible_units))
        asset.execute(target, possible_units)
        self.review()
        


import dataclasses
import enum
from bewe.alpha_seeker import base_data
from typing import Optional


class Action(enum.Enum):
    BUY = 1
    SELL = 2
    HOLD = 3


@dataclasses.dataclass
class Execution:
    action: Action
    confidence: Optional[float] = None
    gain: Optional[float] = None

    def __post_init__(self):
        if self.gain < 1.0:
            raise ValueError(f'gain value should be greater than 1.0, {self.gain} found.')
        if self.confidence < 0.0:
            raise ValueError(f'confidence should be greater than 0.0, {self.confidence} found.')


class Strategy:
    strategy_name = 'Base Strategy'
    required_format = None

    def __post_init__(self):
        if not self.required_format:
            raise ValueError('required_format should be defined.')

    def predict(self, data: base_data.DataUnit) -> Execution:
        raise NotImplementedError
    
    def validate_data(self, data: base_data.DataUnit) -> bool:
        raise NotImplementedError



from setuptools import setup, find_packages
import datetime

version = int(datetime.datetime.now().timestamp())

setup(
    name='bewe',
    version=f'0.1.2.{version}',
    description='The package implements some investment strategies.',
    long_description='Bewe.',
    author='Berlin Hsin',
    author_email='berlin.hsin@gmail.com',
    packages=find_packages(),
    install_requires=[
    ]
)



from bewe.services.crawler import secret_manager
from sqlalchemy import orm
from google.cloud.sql.connector import Connector, IPTypes
from typing import Sequence
import pymysql
import sqlalchemy
import os


DB_NAME_KEY = 'DB_NAME'
DB_USER_KEY = 'DB_USER'
DB_PASS_KEY = 'DB_PASS'
INSTANCE_KEY = 'DB_INSTANCE'


class DBManager:
    def __init__(self):
        self.instance = secret_manager.get_secret_tokens(INSTANCE_KEY)
        self.engine = sqlalchemy.create_engine(
            'mysql+pymysql://',
            creator=self.get_conn
        )

    def get_conn(self) -> pymysql.connections.Connection:
        ip_type = IPTypes.PRIVATE if os.environ.get("PRIVATE_IP") else IPTypes.PUBLIC
        connector = Connector(ip_type)
        conn: pymysql.connections.Connection = connector.connect(
            self.instance,
            'pymysql',
            user=secret_manager.get_secret_tokens(DB_USER_KEY),
            password=secret_manager.get_secret_tokens(DB_PASS_KEY),
            db=secret_manager.get_secret_tokens(DB_NAME_KEY)
        )
        return conn

    def insert(self, records: Sequence[orm.DeclarativeBase]):

        with orm.Session(self.engine) as session:
            for record in records:
                session.add(record)
            session.commit()

    def update(self, records: Sequence[orm.DeclarativeBase]):
        with orm.Session(self.engine) as session:
            for record in records:
                session.merge(record)
            session.commit()

